/**************************************************************************************************
  Filename:       zcl_sampleapps_ui.c
  Revised:        $Date: 2016-8-1 16:04:46 -0700 (Fri, 24 Oct 2014) $
  Revision:       $Revision: 40796 $


  Description:    Z-Stack Sample Application User Interface.


  Copyright 2006-2016 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

/*********************************************************************
  This file implements the user interface that is common to most of the Z-Stack sample applications.

  The sample applications are intended for the following platforms:
  - CC26x2LAUNCHPAD

  The UI peripherals being used:

  - UART based menu system:
    To display only using 3 lines of 18 characters each.

    As input the following keys are used to interact/navigate with the menu system, where applicable:

      - 'a'/'d': depending on the menu screen, these are used for either:
        - move to the previous/next menu screen
        - move to the previous/next digit/sub-item within a menu screen
      - 'a'/'s': change the value of the currently selected item
      - 'e' (Select): execute the operation associated with the current menu screen
      NOTE: the keys can be enter either lower or upper case.

  - Switchs:
    Two switches are used differently by the individual applications. This UI does not use them.

  - LEDs:
    LED1 is used differently by the individual applications.

    LED2:
      - Off: device is not on a network
      - On: device on a network
      - Blinking, 1 second period, 50% duty cycle: device is identifying

  The menu system:

    <HELP> Wellcome screen
      Displays the sample application name.
      Press [OK] to show a simple help screen.

      Help screen
      Press [OK] to go back to the welcome screen.

    <CONFIGURE> Configuration sub-menu
      Press [OK] to display the configuration sub-menu

      <ADD INSTL CODE> install code sub-menu
        Press [OK] to display the install code sub-menu

        <SET INSTL CODE> set install code
          Press [OK] to to edit the install code

          Install code editing screen
            Press [LEFT] / [RIGHT] to select a digit
            Press [UP] / [DOWN] to change the selected digit
            press [OK] to to go back to the higher menu level

        <SET I.C. ADDR> set install code address (coordinator only)
          Press [OK] to to edit the install code address

          Install code address editing screen
            Press [LEFT] / [RIGHT] to select a digit
            Press [UP] / [DOWN] to change the selected digit
            press [OK] to to go back to the higher menu level

        <APLY INST CODE> apply install code
          Press [OK] to apply the address+Install-Code pair (coordinator) or the Install-Code (Routers and End-Devices)
          This menu screen also displays the status of the last apply-install-code operation

        <BACK> go back to higher menu level
          press [OK] to to go back to the higher menu level

      <T.L. TRGT TIME> Set the touchlink enable duration
        Press [UP] / [DOWN] to increase / decrease the displayed time.
        The maximum duration is 86400 seconds. Increasing it above this value will display '(forever)' -
        in this setting, the touchlink target will stay active once started and untill manually stopped.
        The minimum duration is 1 second. Decreasing it below this value sets tohchlink target to always disabled.
        This menu item is only available if BDB_TL_TARGET is defined.

      <T.L. STEALING> enable/disable touchlink stealing
        Press [OK] to toggle (enable/disable) touchlink stealing.
        When stealing is enabled, a touchlink target may be 'stolen' by another device acting as a touchlink
        initiator, which will take it out of its existing network and add it to another network.
        This menu item is only available if BDB_TL_TARGET is defined.

      <T.L. INITIATOR> enable/disable touchlink initiator
        Press [OK] to toggle (enable/disable) touchlink initiator.
        When enabled, touchlink initiator functionality will be executed as part of the commissioning procedure.
        This menu item is only available if BDB_TL_INITIATOR is defined.

      <NWK FORMATION> enable/disable network formation for when cannot connect to an existing network
        Press [OK] to toggle (enable/disable) network formation.
        When enabled, network formation will be executed as part of the commissioning procedure, unless
        the device has already joined a network.

      <NWK STEERING> enable/disable attempting to connect to an existing network
        Press [OK] to toggle (enable/disable) network steering.
        When enabled, network steering will be executed as part of the commissioning procedure, so the
        device will try to join an existing network, and will open the network for joining of other devices
        once it is joined.

      <FINDNG+BINDNG> enable/disable finding and binding
        Press [OK] to toggle (enable/disable) finding and binding (F&B).
        When enabled, finding and binding will be executed as part of the commissioning procedure, so the
        device will either start identifying, look for matchng devices that are currently identifying, or
        both (depending of whether it is a F&B target, initiator, or both).

      <PRI CHANL MASK> set the primary channel mask
        Press [OK] to edit the primary channel list

        Primary channel list editing screen
          Press [LEFT] / [RIGHT] to select a channel (from channel 11 to channel 26)
          Press [UP] / [DOWN] to enable / disable the selected channel, accordingly.
          press [OK] to to go back to the higher menu level

      <SEC CHANL MASK> set the secondary channel mask
        Press [OK] to edit the secondary channel list

        Secondary channel list editing screen
          Press [LEFT] / [RIGHT] to select a channel (from channel 11 to channel 26)
          Press [UP] / [DOWN] to enable / disable the selected channel, accordingly
          press [OK] to to go back to the higher menu level

      <PAN ID> set the PAN ID
        Press [OK] to edit the PAN ID to create / connect to

        PAN ID editing screen
          Press [LEFT] / [RIGHT] to select the digit to edit
          Press [UP] / [DOWN] to change the selected digit
          press [OK] to to go back to the higher menu level

      <BACK> go back to higher menu level
        press [OK] to to go back to the higher menu level

    <COMMISSION> start commissioning
      Press [OK] to start the commissioning procedure. (Note: this is disabled if a previous comissioning is
      already on-going).
      This procedure will execute the following methods, depending on whether they are enabled or disabled
      in the respective configuration item):
        - Touchlink (as Touchlink Initiator)
        - Network Steering
        - Network Formation
        - Finding and Binding
      This screen shows the following information
        Line1:
          Current commissioning method being executed:
            TL: Touchlink
            NS: Network Steering
            NF: Network Formation
            FB: Finding And Binding
            PL: Parent Lost (for end devices only)
            -- - idle (commissioning not currently active)
          Network status
            NotOnNwk - not currently connected to a network
            FORM - network was formed by the current device during the latest execution of the NF method
            JOIN - the current device joined an existing network during the latest execution of the NS method
            TCHL - the current device joined a network using touchlink
            EXST - the device was already connected to a network when the commissioning was started
          Joining permission state (not showing on end devices):
            CLOSED - the current device is closed for joining of other devices
            OpenXXX - the current device is open for joining of other devices, and will close in XXX seconds
        Line2:
          IdXXX - The device is identifying (if XXX > 0), and will stop identifying in XXX seconds.
          SrchXXX/YY - The device is currently performing F&B as an Initiator (if XXX > 0), and will stop in
            XXX seconds. YY is the number of matching endpoints that were found, for which bindings were
            successfully created or already existed.

    <T.L. TARGET> start touchlink target
      Press [OK] to start / stop touchlink target functionality.
      When not active, pressing [OK] will start touchlink target for the duration defined by the respective
      configuration item.
      When active, pressing OK will immediately stop touchlink target functionality.
      This menu item is only available if BDB_TL_TARGET is defined.

    <APP MENU> application-specific sub-menu
      Press [OK] to enter the application-specific sub-menu.
      See the main comment in the specific application c file for more details.

    <INFO> device and network information
      This screen has no operation related to it. It shows the following information:
        Line1:
          Device IEEE address
        Line2: (only meaningful when the device is on a network)
          nXXXX - the Network pan-id
          cXX - the Channel the network is active on
          aXXXX - the short Address of the device
        Line3:
          XXX (YYY) - the device is not connected to a network.
            XXX is either ZR, ZC or ZED, depending on the build configuration.
            YYY is the network-state value (see definition of devStates_t in ZDApp.h).
          ZC - the device is connected to a network as a coordinator.
          ZR CENT - the device is connected to a centralized network as a router.
          ZR DIST - the device is connected to a distributed network as a router.
          ZED pXXXX - the device is connected to a network as an end device. Parent address is XXXX.

    <RESET TO FN> reset the device to factory-new
      Press [OK] to reset the device to factory-new. The device will disconnect from the network, and
      all configurations and network information will be deleted.

*********************************************************************/

#if (BDB_INSTALL_CODE_USE!=BDB_INSTALL_CODE_USE_IC_CRC)
#warning Install-Code functionality is disabled. To enable, make sure BDB_INSTALL_CODE_USE==BDB_INSTALL_CODE_USE_IC_CRC
#endif

/*********************************************************************
 * INCLUDES
 */
#include "rom_jt_154.h"

#include "hal_types.h"
#include "hal_key.h"
#include "bdb_interface.h"
#include "zcl.h"
#include "zd_app.h"
#include "zcl_sampleapps_ui.h"

#include <ti/sysbios/knl/Semaphore.h>
#include "util_timer.h"

#ifdef BOARD_DISPLAY_USE_UART
#include <ti/drivers/UART.h>
#include "Board.h"
#endif

#include "board_led.h"
#include <string.h>

#ifdef BDB_TL_TARGET
  #include "bdb_touchlink_target.h"
  #include "touchlink_target_app.h"
#endif

#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
#include "gp_common.h"
#endif

#ifdef BOARD_DISPLAY_USE_UART
    #define USE_ZCL_SAMPLEAPP_UI
#endif


#ifdef USE_ZCL_SAMPLEAPP_UI

/*********************************************************************
 * CONSTANT-MACROS AND ENUMERATIONS
 */
enum
{
  UI_STATE_DEFAULT = 0,
  UI_STATE_CONFIGURE,
  UI_STATE_COMMISSION,
#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
  UI_STATE_GPP_COMMISSION,
#endif
#ifdef BDB_TL_TARGET
  UI_STATE_TOGGLE_TL_TARGET,
#endif
  UI_STATE_APP_SPECIFIC_MENU,
  UI_STATE_INFO,
  UI_STATE_RESET_TO_FACTORY_NEW,
  UI_STATE_STATE_HELP,
  UI_STATE_SET_INST_CODE,
#ifdef BDB_TL_TARGET
  UI_STATE_TOUCHLINK_TARGET,
  UI_STATE_TOUCHLINK_STEALING,
#endif
#ifdef BDB_TL_INITIATOR
  UI_STATE_TOUCHLINK_INITIATOR,
#endif
  UI_STATE_NETWORK_FORMATION,
  UI_STATE_NETWORK_STEERING,
  UI_STATE_FINDING_AND_BINDING,
  UI_STATE_SET_PRI_CHANEL_MASK_0,
  UI_STATE_SET_SEC_CHANEL_MASK_0,
  UI_STATE_SET_PAN_ID_0,
  UI_STATE_BACK_FROM_CONFIGURE,
  UI_STATE_SET_PRI_CHANEL_MASK_1,
  UI_STATE_SET_SEC_CHANEL_MASK_1,
  UI_STATE_SET_PAN_ID_1,
  UI_STATE_SET_INST_CODE_0,
#if (ZG_BUILD_COORDINATOR_TYPE)
  UI_STATE_SET_INST_CODE_ADDR_0,
#endif
  UI_STATE_SET_INST_CODE_DONE,
  UI_STATE_BACK_FROM_INSTALL_CODE,
  UI_STATE_SET_INST_CODE_1,
#if (ZG_BUILD_COORDINATOR_TYPE)
  UI_STATE_SET_INST_CODE_ADDR_1,
#endif
};


#if ZG_BUILD_COORDINATOR_TYPE
//By default, Coordiantor has Formation selected in the UI menu
#define DEFAULT_COMISSIONING_MODE (BDB_COMMISSIONING_MODE_NWK_STEERING | BDB_COMMISSIONING_MODE_NWK_FORMATION | BDB_COMMISSIONING_MODE_FINDING_BINDING)
#else
//By default, joining devices such as Router and ZED do not have formation selected. It can be enabled in the UI if needed.
#define DEFAULT_COMISSIONING_MODE (BDB_COMMISSIONING_MODE_NWK_STEERING | BDB_COMMISSIONING_MODE_FINDING_BINDING)
#endif


#define TOUCHLINK_TIMEOUT_INCREMENTS_INITIAL 1000

#define DEFAULT_TOUCHLINK_TARGET_TIMEOUT 30000

#define UI_CURSOR_REFRESH_INTERVAL 500

#define GPP_TIMEOUT_UPDATE_INTERVAL 1000

#define UI_AUTO_REFRESH_INTERVAL_ON_COMISSIONING_SCREEN 250

#define UI_AUTO_REFRESH_INTERVAL_ON_TOUCHLINK_TARGET_SCREEN 1000
#define UI_AUTO_REFRESH_INTERVAL_ON_TOUCHLINK_TARGET_SCREEN_END 50


#define UI_DISPLAY_LINE_LENGTH 16



#define STATUS_UNKNOWN 0xFF

#define UI_CURSOR_CHAR ' '

#define UI_INSTALL_CODE_DEFAULT {0x83,0xFE,0xD3,0x40,0x7A,0x93,0x97,0x23,0xA5,0xC6,0x39,0xB2,0x69,0x16,0xD5,0x05,0xC3,0xB5} //This install code produces the key: 66B6900981E1EE3CA4206B6B861C02BB
#define UI_INSTALL_CODE_ADDR_DEFAULT {0xE7,0xFC,0x0E,0x04,0x00,0x4B,0x12,0x00}



#define UI_DISPLAY_LINE_1 0x01
#define UI_DISPLAY_LINE_2 0x02
#define UI_DISPLAY_LINE_3 0x03

#define UI_DISPLAY_CC2538_PREFIX_COUNT 0



#define UI_LED_COORDINATOR_DUTY_CYCLE 75
#define UI_LED_ROUTER_DUTY_CYCLE      95
#define UI_LED_END_DEVICE_DUTY_CYCLE 100

#define UI_LED_DEVICE_STATUS         board_led_type_LED2


#define NWK_OPEN_FOR_JOINING 0x1
#define NWK_OPEN_TOUCHLINK_AS_TARGET 0x2

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
* LOCAL FUNCTIONS DECLARATIONS
*/
static void uiUintToString(uint32_t value, char * str, uint8_t base, uint8_t num_of_digists, bool pad0, bool reverse);
static void uiArrayToString(uint8_t * buf, char * str, uint8_t num_of_digists, bool big_endian);
static void uiCreateChannelMaskString(uint32_t channel_mask, char * str);

#ifdef BOARD_DISPLAY_USE_UART
static void UartWriteCallback(UART_Handle handle, void *buf, size_t count);
static void UartReadCallback(UART_Handle handle, void *buf, size_t size);
#endif

static void display_writeString(void * buffer, size_t size, uint8_t line);

#ifdef BDB_TL_TARGET
static void uiActionToggleTlTarget(uint16_t keys);
#endif

static void uiActionSetInstallCode(uint16_t keys);
static void uiActionResetToFactoryNew(uint16_t keys);
static void uiActionConfigureEnables(uint16_t keys);
static void uiActionMenuJump(uint16_t keys);
static void uiActionProcessConfigureChannels(uint16_t keys);
static void uiActionProcessConfigurePanId(uint16_t keys);
static void uiActionProcessPrepareInstallCode(uint16_t keys);

static void zclSampleApps_initializeClocks(void);

/*********************************************************************
 * CONSTANTS
 */
static const uiState_t gui_states_main[] =
{
  /*  UI_STATE_DEFAULT                */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_RESET_TO_FACTORY_NEW,    UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
  /*  UI_STATE_CONFIGURE              */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
  /*  UI_STATE_COMMISSION             */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &UI_ActionStartComissioning},
#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
  /*  UI_STATE_GPP_COMMISSION         */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            0, NULL},
#endif
#ifdef BDB_TL_TARGET
  /*  UI_STATE_TOGGLE_TL_TARGET       */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionToggleTlTarget},
#endif
  /*  UI_STATE_APP_SPECIFIC_MENU      */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &UI_ActionAppSpecificMenu},
  /*  UI_STATE_INFO                   */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            0, NULL},
  /*  UI_STATE_RESET_TO_FACTORY_NEW   */ {UI_STATE_DEFAULT,               UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionResetToFactoryNew},

  /*  UI_STATE_STATE_HELP             */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_5_PRESSED,&uiActionMenuJump},

  /*  UI_STATE_SET_INST_CODE          */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_BACK_FROM_CONFIGURE,     UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
#ifdef BDB_TL_TARGET
  /*  UI_STATE_TOUCHLINK_TARGET       */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED , &uiActionConfigureEnables},
  /*  UI_STATE_TOUCHLINK_STEALING     */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionConfigureEnables},
#endif
#ifdef BDB_TL_INITIATOR
  /*  UI_STATE_TOUCHLINK_INITIATOR    */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionConfigureEnables},
#endif
  /*  UI_STATE_NETWORK_FORMATION      */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionConfigureEnables},
  /*  UI_STATE_NETWORK_STEERING       */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionConfigureEnables},
  /*  UI_STATE_FINDING_AND_BINDING    */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionConfigureEnables},
  /*  UI_STATE_SET_PRI_CHANEL_MASK_0  */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
  /*  UI_STATE_SET_SEC_CHANEL_MASK_0  */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
  /*  UI_STATE_SET_PAN_ID_0           */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
  /*  UI_STATE_BACK_FROM_CONFIGURE    */ {UI_STATE_SET_INST_CODE,         UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},

  /*  UI_STATE_SET_PRI_CHANEL_MASK_1  */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED | UI_KEY_SW_5_PRESSED | UI_KEY_SW_2_PRESSED  | UI_KEY_SW_4_PRESSED , &uiActionProcessConfigureChannels},
  /*  UI_STATE_SET_SEC_CHANEL_MASK_1  */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED | UI_KEY_SW_5_PRESSED | UI_KEY_SW_2_PRESSED  | UI_KEY_SW_4_PRESSED , &uiActionProcessConfigureChannels},
  /*  UI_STATE_SET_PAN_ID_1           */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED | UI_KEY_SW_5_PRESSED | UI_KEY_SW_2_PRESSED  | UI_KEY_SW_4_PRESSED , &uiActionProcessConfigurePanId},

  /*  UI_STATE_SET_INST_CODE_0        */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_BACK_FROM_INSTALL_CODE,  UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
#if (ZG_BUILD_COORDINATOR_TYPE)
  /*  UI_STATE_SET_INST_CODE_ADDR_0   */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},
#endif
  /*  UI_STATE_SET_INST_CODE_DONE     */ {UI_STATE_DEFAULT_MOVE,          UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionSetInstallCode},
  /*  UI_STATE_BACK_FROM_INSTALL_CODE */ {UI_STATE_SET_INST_CODE_0,       UI_STATE_DEFAULT_MOVE,            UI_KEY_SW_5_PRESSED, &uiActionMenuJump},

  /*  UI_STATE_SET_INST_CODE_1        */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED | UI_KEY_SW_5_PRESSED | UI_KEY_SW_2_PRESSED  | UI_KEY_SW_4_PRESSED , &uiActionProcessPrepareInstallCode},
#if (ZG_BUILD_COORDINATOR_TYPE)
  /*  UI_STATE_SET_INST_CODE_ADDR_1   */ {UI_STATE_UNCHANGED,             UI_STATE_UNCHANGED,               UI_KEY_SW_1_PRESSED | UI_KEY_SW_3_PRESSED | UI_KEY_SW_5_PRESSED | UI_KEY_SW_2_PRESSED  | UI_KEY_SW_4_PRESSED , &uiActionProcessPrepareInstallCode},
#endif
};


#ifdef BOARD_DISPLAY_USE_UART
//Size of the displayed caracters + clear screen + 2 new lines + 2 carriage return
char LineUARTBuf[UI_DISPLAY_LINE_LENGTH*3 + 1 + 2 + 2];
char * line[3];
#endif
/*********************************************************************
 * GLOBAL VARIABLES
 */

#ifdef BOARD_DISPLAY_USE_UART
UART_Params uartParams;
UART_Handle Uhandle=NULL;
uint8_t uartTxBuffer[1];
uint8_t uartRxBuffer[1];
uint8_t totalRxBytes=sizeof(uartTxBuffer);
#endif

/*********************************************************************
 * GLOBAL FUNCTIONS
 */

/*********************************************************************
 * EXTERNAL REFERENCES
 */
extern uint8_t aExtendedAddress[];

/*********************************************************************
 * LOCAL VARIABLES
 */
static uint16_t uiAutoRefreshEvent;
static uint16_t uiUartProcessEvent;
#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
static uint16_t uigppProcessEvent = 0;
static uint16_t gppCommissioningTimeout = 0;
#endif
static uint16_t * pUiIdentifyTimeAttribute;
static uiAppFNReset_t uiAppFNReset = NULL;
static uiRefresh_t uiRefresh;
static const uiState_t * uiAppStatesMain = NULL;

static uint8_t FBMatchesFound = 0;

static int uiCurrentState = UI_STATE_DEFAULT;
static bool uiCommissioningIsInitializing = FALSE;

static uint8_t uiCursorLine = 0xFF;
static uint8_t uiCursorCol;
static uint8_t uiCursorState;

static uint8_t * pUiSelectedBdbComissioningModes;

static ZStatus_t uiAddInstallCodeLastStatus = STATUS_UNKNOWN;

static char * uiCommissioningStateStr = "--";
static char * uiCommissioningNetworkConnectionStr = NULL;

static bool uiResetInitiated = FALSE;

#ifdef BDB_TL_TARGET
static uint32_t uiAutoRefreshInterval;
static uint32_t uiTouchlinkTargetTimeout = DEFAULT_TOUCHLINK_TARGET_TIMEOUT;
#endif

static uint8_t uiInstallCode[] = UI_INSTALL_CODE_DEFAULT;
#if (ZG_BUILD_COORDINATOR_TYPE)
static uint8_t uiInstallCodeAddr[Z_EXTADDR_LEN] = UI_INSTALL_CODE_ADDR_DEFAULT;
#endif

static const uiState_t * uiStates = gui_states_main;

static char * uiAppTitleStr;

static zstack_DevState uiNwkStateShadow = zstack_DevState_HOLD;

static Clock_Struct uiAppClkStruct;
static uint32_t *uiAppEvents;
static Semaphore_Handle uiAppSem;
static Clock_Handle uiAppClkHandle;
static uint8_t  uiAppEntity;

#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
static Clock_Struct uiGppClkStruct;
static Clock_Handle uiGppClkHandle;
#endif

/*********************************************************************
 * LOCAL UTILITY FUNCTIONS
 */

/*********************************************************************
 * @fn          uiUintToString
 *
 * @brief       format an integer into a string buffer.
 *
 * @param       value - 32bit unsigned int value to be formatted
 *              str - pointer to a buffer to store the formatted bnumber
 *              base - base represenation of the value. currently only tested base 10 and 16
 *              num_of_digists - number of digits to include in the formatted string
 *              pad0 - should be set to TRUE to pad the number with leading 0's as required
 *              reverse - should be set to TRUE to reverse the output string
 *
 * @return      none
 */
static void uiUintToString (uint32_t value, char * str, uint8_t base, uint8_t num_of_digists, bool pad0, bool reverse)
{
  int i;
  uint8_t index;

  for (i = 0; i < num_of_digists; i++)
  {
    index = (reverse ? i : num_of_digists - 1 - i);
    str[index] = '0' + (value % base);
    if (str[index] > '9')
    {
      str[index] += 'A' - '0' - 10;
    }
    value /= base;
    if ((!pad0) && (value == 0))
    {
      break;
    }
  }
}

/*********************************************************************
 * @fn          uiArrayToString
 *
 * @brief       format a memory buffer into a string buffer in hex representation.
 *
 * @param       buf - pointer to a bufer to be formatted
 *              str - pointer to a buffer to store the formatted string
 *              num_of_digists - number of digits to include in the formatted string
 *              big_endian - whether the memory content should be represented as big or little endian
 *
 * @return      none
 */
static void uiArrayToString (uint8_t * buf, char * str, uint8_t num_of_digists, bool big_endian)
{
  int i;
  uint8_t stringIndex;
  uint8_t value;

  for (i = 0; i < num_of_digists; i++)
  {
    stringIndex = (big_endian ? i : num_of_digists - 1 - i);
    if(big_endian)
    {
      value = (buf[i / 2] >> (4 * (!(i % 2)))) & 0x0F;
    }
    else
    {
      value = (buf[i / 2] >> (4 * (i % 2))) & 0x0F;
    }
    str[stringIndex] = '0' + value;
    if (str[stringIndex] > '9')
    {
      str[stringIndex] += 'A' - '0' - 10;
    }
  }
}

/*********************************************************************
 * @fn          uiCreateChannelMaskString
 *
 * @brief       format a channel mask into a string bugger
 *
 * @param       channel_mask - bitmask of the enabled / disabled channels (bits 11-26 represent the
 *                respective channels)
 *              str - pointer to a buffer to store the formatted string
 *
 * @return      none
 */
static void uiCreateChannelMaskString(uint32_t channel_mask, char * str)
{
  int i;
  uint32_t mask = 0x00000800;

  for (i = 0; i < 16; i++)
  {
    str[i] = channel_mask & mask ? '*' : '-';
    mask <<= 1;
  }

  str[16] = 0;
}


/*********************************************************************
 * LOCAL FUNCTIONS
 */

/*********************************************************************
 * @fn          uiActionMenuJump
 *
 * @brief       State-machine action for jumping to another state
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionMenuJump(uint16_t keys)
{
  switch (uiCurrentState)
  {
    case UI_STATE_DEFAULT:
      uiCurrentState = UI_STATE_STATE_HELP;
      break;
    case UI_STATE_STATE_HELP:
      uiCurrentState = UI_STATE_DEFAULT;
      break;
    case UI_STATE_CONFIGURE:
      uiCurrentState = UI_STATE_SET_INST_CODE;
      break;
    case UI_STATE_BACK_FROM_CONFIGURE:
      uiCurrentState = UI_STATE_CONFIGURE;
      break;
    case UI_STATE_SET_PRI_CHANEL_MASK_0:
      uiCurrentState = UI_STATE_SET_PRI_CHANEL_MASK_1;
      break;
    case UI_STATE_SET_SEC_CHANEL_MASK_0:
      uiCurrentState = UI_STATE_SET_SEC_CHANEL_MASK_1;
      break;
    case UI_STATE_SET_PAN_ID_0:
      uiCurrentState = UI_STATE_SET_PAN_ID_1;
      break;
    case UI_STATE_SET_INST_CODE:
      uiCurrentState = UI_STATE_SET_INST_CODE_0;
      break;
    case UI_STATE_SET_INST_CODE_0:
      uiCurrentState = UI_STATE_SET_INST_CODE_1;
      break;
#if (ZG_BUILD_COORDINATOR_TYPE)
    case UI_STATE_SET_INST_CODE_ADDR_0:
      uiCurrentState = UI_STATE_SET_INST_CODE_ADDR_1;
      break;
#endif
    case UI_STATE_BACK_FROM_INSTALL_CODE:
      uiCurrentState = UI_STATE_SET_INST_CODE;
      break;
  }
}

/*********************************************************************
 * @fn          UI_ActionStartComissioning
 *
 * @brief       State-machine action for starting commissioning
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
void UI_ActionStartComissioning(uint16_t keys)
{
  //set commissioning state in case called from outside of UI
  uiCurrentState = UI_STATE_COMMISSION;

  // reset network connection string before commissioning again
  uiCommissioningNetworkConnectionStr = NULL;

  if (*(pUiSelectedBdbComissioningModes) != 0)
  {
    zstack_bdbStartCommissioningReq_t zstack_bdbStartCommissioningReq;

    FBMatchesFound = 0;

    // update UI now, since Zstackapi_bdbStartCommissioningReq() is blocking for a few seconds,
    // and we want to give a prompt response to the user
    uiCommissioningIsInitializing = TRUE;
    UI_Refresh();

    //if already on the network - mark it as 'existing connection'
    if ((uiNwkStateShadow == zstack_DevState_DEV_END_DEVICE) || (uiNwkStateShadow == zstack_DevState_DEV_ZB_COORD) || (uiNwkStateShadow == zstack_DevState_DEV_ROUTER))
    {
      uiCommissioningNetworkConnectionStr = "EXST";
    }
    zstack_bdbStartCommissioningReq.commissioning_mode = *(pUiSelectedBdbComissioningModes);

    Zstackapi_bdbStartCommissioningReq(uiAppEntity,&zstack_bdbStartCommissioningReq);
  }
}

/*********************************************************************
 * @fn          UI_actionConfigureChannels
 *
 * @brief       Set channel mask in UI
 *
 * @param       channelMask - channel mask
 *
 * @return      none
 */
void UI_actionConfigureChannels(uint32_t channelMask)
{
    bdbAttributes.bdbPrimaryChannelSet = channelMask;

    uiCurrentState = UI_STATE_SET_PRI_CHANEL_MASK_0;

    return;
}

/*********************************************************************
 * @fn          UI_ActionConfigurePanId
 *
 * @brief       External action to configuring the PAN ID
 *
 * @param       panID - PANID to set
 *
 * @return      none
 */
void UI_ActionConfigurePanId(uint16_t panID)
{
    zstack_sysConfigWriteReq_t writeReq = {0};

    zgConfigPANID = panID;

    // Update the config PAN ID, defined in znwk_config.h
    writeReq.has_panID = true;
    writeReq.panID = zgConfigPANID;
    (void)Zstackapi_sysConfigWriteReq(uiAppEntity, &writeReq);

    uiCurrentState = UI_STATE_SET_PAN_ID_0;
}

/*********************************************************************
 * @fn          uiActionSetInstallCode
 *
 * @brief       State-machine action for setting the install code
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionSetInstallCode(uint16_t keys)
{
  zstack_bdbGenerateInstallCodeCRCReq_t zstack_bdbGenerateInstallCodeCRCReq;
  zstack_bdbGenerateInstallCodeCRCRsp_t zstack_bdbGenerateInstallCodeCRCRsp;

  OsalPort_memcpy(zstack_bdbGenerateInstallCodeCRCReq.installCode,uiInstallCode,INSTALL_CODE_LEN);
  Zstackapi_bdbGenerateInstallCodeCRCReq(uiAppEntity,&zstack_bdbGenerateInstallCodeCRCReq,
                                                      &zstack_bdbGenerateInstallCodeCRCRsp);

  uiInstallCode[INSTALL_CODE_LEN] = zstack_bdbGenerateInstallCodeCRCRsp.CRC & 0xFF;
  uiInstallCode[INSTALL_CODE_LEN + 1] = zstack_bdbGenerateInstallCodeCRCRsp.CRC >> 8;

#if (ZG_BUILD_COORDINATOR_TYPE)
  if (ZG_DEVICE_COORDINATOR_TYPE)
  {
      zstack_bdbAddInstallCodeReq_t zstack_bdbAddInstallCodeReq;

    memset(&zstack_bdbAddInstallCodeReq,0,sizeof(zstack_bdbAddInstallCodeReq_t));
    OsalPort_memcpy(zstack_bdbAddInstallCodeReq.pExt, uiInstallCodeAddr, Z_EXTADDR_LEN);
    OsalPort_memcpy(zstack_bdbAddInstallCodeReq.pInstallCode, uiInstallCode, INSTALL_CODE_LEN + INSTALL_CODE_CRC_LEN);

    uiAddInstallCodeLastStatus = Zstackapi_bdbAddInstallCodeReq(uiAppEntity, &zstack_bdbAddInstallCodeReq);
  }
  else
  {
      zstack_bdbSetActiveCentralizedLinkKeyReq_t zstack_bdbSetActiveCentralizedLinkKeyReq;

      memset(&zstack_bdbSetActiveCentralizedLinkKeyReq,0, sizeof(zstack_bdbSetActiveCentralizedLinkKeyReq_t));

      zstack_bdbSetActiveCentralizedLinkKeyReq.zstack_CentralizedLinkKeyModes = zstack_UseInstallCode;
      OsalPort_memcpy(zstack_bdbSetActiveCentralizedLinkKeyReq.pKey,uiInstallCode, INSTALL_CODE_LEN + INSTALL_CODE_CRC_LEN);

      uiAddInstallCodeLastStatus = Zstackapi_bdbSetActiveCentralizedLinkKeyReq(uiAppEntity, &zstack_bdbSetActiveCentralizedLinkKeyReq);

  }
#else
      zstack_bdbSetActiveCentralizedLinkKeyReq_t zstack_bdbSetActiveCentralizedLinkKeyReq;

      zstack_bdbSetActiveCentralizedLinkKeyReq.zstack_CentralizedLinkKeyModes = zstack_UseInstallCode;

      zstack_bdbSetActiveCentralizedLinkKeyReq.pKey = OsalPort_malloc(INSTALL_CODE_LEN + INSTALL_CODE_CRC_LEN);

      OsalPort_memcpy(zstack_bdbSetActiveCentralizedLinkKeyReq.pKey,uiInstallCode,INSTALL_CODE_LEN + INSTALL_CODE_CRC_LEN);

      uiAddInstallCodeLastStatus = Zstackapi_bdbSetActiveCentralizedLinkKeyReq(uiAppEntity, &zstack_bdbSetActiveCentralizedLinkKeyReq);

      OsalPort_free(zstack_bdbSetActiveCentralizedLinkKeyReq.pKey);
#endif
}

/*********************************************************************
 * @fn          UI_ActionAppSpecificMenu
 *
 * @brief       State-machine action for executing the application-specific sub-menus
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
void UI_ActionAppSpecificMenu(uint16_t keys)
{
  if (uiAppStatesMain != NULL)
  {
    uiStates = uiAppStatesMain;
    uiCurrentState = 1; //do not start from 0, which is the 'back' menu item
  }
}

/*********************************************************************
 * @fn          UI_ActionBackFromAppMenu
 *
 * @brief       Application state-machine action for returning to the common menu
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
void UI_ActionBackFromAppMenu(uint16_t keys)
{
  uiStates = gui_states_main;
  uiCurrentState = UI_STATE_APP_SPECIFIC_MENU;
}

#ifdef BDB_TL_TARGET
/*********************************************************************
 * @fn          uiActionToggleTlTarget
 *
 * @brief       State-machine action for toggling touchlink-target functionality
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
void uiActionToggleTlTarget(uint16_t keys)
{
  if (!touchLinkTargetEnabled)
  {
    if (uiTouchlinkTargetTimeout > 0)
    {
      zstack_bdbTouchLinkTargetEnableCommissioningReq_t zstack_bdbTouchLinkTargetEnableCommissioningReq;
      zstack_bdbTouchLinkTargetEnableCommissioningReq.timeoutTime = uiTouchlinkTargetTimeout;

      Zstackapi_bdbTouchLinkTargetEnableCommissioningReq(uiAppEntity, &zstack_bdbTouchLinkTargetEnableCommissioningReq);

      uiAutoRefreshInterval = UI_AUTO_REFRESH_INTERVAL_ON_TOUCHLINK_TARGET_SCREEN;
    }
  }
  else
  {
    Zstackapi_bdbTouchLinkTargetDisableCommissioningReq(uiAppEntity);

    uiAutoRefreshInterval = UI_AUTO_REFRESH_INTERVAL_ON_TOUCHLINK_TARGET_SCREEN_END;
  }
}
#endif

/*********************************************************************
 * @fn          uiActionProcessConfigureChannels
 *
 * @brief       State-machine action for configuring channel masks
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionProcessConfigureChannels(uint16_t keys)
{
  uint32_t * channelMask;

  uiCursorLine = 1;
  uiCursorState = 0;

  if (uiCurrentState == UI_STATE_SET_PRI_CHANEL_MASK_1)
  {
    channelMask = &bdbAttributes.bdbPrimaryChannelSet;
  }
  else
  {
    channelMask = &bdbAttributes.bdbSecondaryChannelSet;
  }

  if (keys & UI_KEY_SW_5_PRESSED)
  {
    uiCursorCol = 0;
  }

  if (keys & UI_KEY_SW_1_PRESSED)
  {
    *channelMask |= ((uint32_t)0x00000800 << uiCursorCol);
    uiCursorState = 1;
  }
  else if (keys & UI_KEY_SW_3_PRESSED)
  {
    *channelMask &= ~(((uint32_t)0x00000800 << uiCursorCol));
    uiCursorState = 1;
  }
  else if (keys & UI_KEY_SW_2_PRESSED)
  {
    if (uiCursorCol < 15)
    {
      uiCursorCol++;
    }
    else
    {
      uiCursorCol = 0;
    }
  }
  else if (keys & UI_KEY_SW_4_PRESSED)
  {
    if (uiCursorCol > 0)
    {
      uiCursorCol--;
    }
    else
    {
      uiCursorCol = 15;
    }
  }
  else if (keys & UI_KEY_SW_5_PRESSED)
  {
    if (uiCurrentState == UI_STATE_SET_PRI_CHANEL_MASK_1)
    {
      uiCurrentState = UI_STATE_SET_PRI_CHANEL_MASK_0;
    }
    else
    {
      uiCurrentState = UI_STATE_SET_SEC_CHANEL_MASK_0;
    }

    uiCursorLine = 0xFF;
  }
}

/*********************************************************************
 * @fn          uiActionProcessConfigurePanId
 *
 * @brief       State-machine action for configuring the PAN ID
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionProcessConfigurePanId(uint16_t keys)
{
  uint8_t shift;
  uint8_t digit;

  uiCursorLine = 0;
  uiCursorState = 0;

  if (keys & UI_KEY_SW_5_PRESSED)
  {
    uiCursorCol = 6;
  }

  if ((keys & UI_KEY_SW_1_PRESSED) || (keys & UI_KEY_SW_3_PRESSED))
  {
    shift = 4 * (9 - uiCursorCol);
    digit = (zgConfigPANID >> shift) & 0xF;
    if (keys & UI_KEY_SW_1_PRESSED)
    {
      if (digit < 15)
      {
        digit++;
      }
      else
      {
        digit = 0;
      }
    }
    else if (keys & UI_KEY_SW_3_PRESSED)
    {
      if (digit > 0)
      {
        digit--;
      }
      else
      {
        digit = 15;
      }
    }

    zgConfigPANID &= ~((uint32_t)0xF << shift);
    zgConfigPANID |= (uint32_t)digit << shift;
    uiCursorState = 1;
  }
  else if (keys & UI_KEY_SW_2_PRESSED)
  {
    if (uiCursorCol < 9)
    {
      uiCursorCol++;
    }
  }
  else if (keys & UI_KEY_SW_4_PRESSED)
  {
    if (uiCursorCol > 6)
    {
      uiCursorCol--;
    }
  }
  else if (keys & UI_KEY_SW_5_PRESSED)
  {
      zstack_sysConfigWriteReq_t writeReq = {0};

      // Update the config PAN ID, defined in znwk_config.h
      writeReq.has_panID = true;
      writeReq.panID = zgConfigPANID;
      (void)Zstackapi_sysConfigWriteReq(uiAppEntity, &writeReq);

    uiCurrentState = UI_STATE_SET_PAN_ID_0;

    uiCursorLine = 0xFF;
  }
}

/*********************************************************************
 * @fn          uiActionProcessPrepareInstallCode
 *
 * @brief       State-machine action for editing the install-code and the install-code address
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionProcessPrepareInstallCode(uint16_t keys)
{
  static uint8_t * pCurrentModifiedField;
  static uint8_t CurrentModifiedFieldLen;
  static bool CurrentModifiedFieldIsBigEndian;

  static uint8_t index;

  uint8_t shift;
  uint8_t digit;
  uint8_t bufIndex;

  uiCursorState = 0;

  if (keys & UI_KEY_SW_5_PRESSED)
  {
    index = 0;

#if (ZG_BUILD_COORDINATOR_TYPE)
    if (uiCurrentState == UI_STATE_SET_INST_CODE_ADDR_1)
    {
      pCurrentModifiedField = uiInstallCodeAddr;
      CurrentModifiedFieldLen = sizeof(uiInstallCodeAddr);
      CurrentModifiedFieldIsBigEndian = FALSE;
    }
    else
#endif
    {
      zstack_bdbGenerateInstallCodeCRCReq_t zstack_bdbGenerateInstallCodeCRCReq;
      zstack_bdbGenerateInstallCodeCRCRsp_t zstack_bdbGenerateInstallCodeCRCRsp;

      OsalPort_memcpy(zstack_bdbGenerateInstallCodeCRCReq.installCode,uiInstallCode,INSTALL_CODE_LEN);

      pCurrentModifiedField = uiInstallCode;
      CurrentModifiedFieldLen = INSTALL_CODE_LEN;
      CurrentModifiedFieldIsBigEndian = TRUE;

      Zstackapi_bdbGenerateInstallCodeCRCReq(uiAppEntity,&zstack_bdbGenerateInstallCodeCRCReq,
                                                   &zstack_bdbGenerateInstallCodeCRCRsp);

      uiInstallCode[INSTALL_CODE_LEN] = zstack_bdbGenerateInstallCodeCRCRsp.CRC  & 0xFF;
      uiInstallCode[INSTALL_CODE_LEN + 1] = zstack_bdbGenerateInstallCodeCRCRsp.CRC  >> 8;
    }
  }

  if ((keys & UI_KEY_SW_1_PRESSED) || (keys & UI_KEY_SW_3_PRESSED))
  {
#if (ZG_BUILD_COORDINATOR_TYPE)
    if (uiCurrentState == UI_STATE_SET_INST_CODE_ADDR_1)
    {
      pCurrentModifiedField = uiInstallCodeAddr;
      CurrentModifiedFieldLen = sizeof(uiInstallCodeAddr);
      CurrentModifiedFieldIsBigEndian = FALSE;
    }
    else
#endif
    {
        pCurrentModifiedField = uiInstallCode;
        CurrentModifiedFieldLen = INSTALL_CODE_LEN;
        CurrentModifiedFieldIsBigEndian = TRUE;
    }
    if(pCurrentModifiedField == uiInstallCode)
    {
      shift = 4 * ((CurrentModifiedFieldIsBigEndian ? (index + 1) : index ) % 2);
    }
    else
    {
      shift = 4 * ((CurrentModifiedFieldIsBigEndian ? index : (index + 1)) % 2);
    }
    bufIndex = (CurrentModifiedFieldIsBigEndian ? (index / 2) : (CurrentModifiedFieldLen - 1 - (index / 2)));
    digit = (pCurrentModifiedField[bufIndex] >> shift) & 0xF;

    if (keys & UI_KEY_SW_1_PRESSED)
    {
      if (digit < 15)
      {
        digit++;
      }
      else
      {
        digit = 0;
      }
    }
    else if (keys & UI_KEY_SW_3_PRESSED)
    {
      if (digit > 0)
      {
        digit--;
      }
      else
      {
        digit = 15;
      }
    }

    pCurrentModifiedField[bufIndex] &= ~((uint32_t)0xF << shift);
    pCurrentModifiedField[bufIndex] |= (uint32_t)digit << shift;
    uiCursorState = 1;
  }
  else if (keys & UI_KEY_SW_2_PRESSED)
  {
    if (index < CurrentModifiedFieldLen * 2 - 1)
    {
      index++;
    }
    else
    {
      index = 0;
    }
  }
  else if (keys & UI_KEY_SW_4_PRESSED)
  {
    if (index > 0)
    {
      index --;
    }
    else
    {
      index = (CurrentModifiedFieldLen * 2 - 1);
    }
  }

  uiCursorLine = index / UI_DISPLAY_LINE_LENGTH;
  uiCursorCol = index % UI_DISPLAY_LINE_LENGTH;

  if ((uiCurrentState == UI_STATE_SET_INST_CODE_1) && ((keys & UI_KEY_SW_1_PRESSED) || (keys & UI_KEY_SW_3_PRESSED)))
  {
    zstack_bdbGenerateInstallCodeCRCReq_t zstack_bdbGenerateInstallCodeCRCReq;
    zstack_bdbGenerateInstallCodeCRCRsp_t zstack_bdbGenerateInstallCodeCRCRsp;

    OsalPort_memcpy(zstack_bdbGenerateInstallCodeCRCReq.installCode,uiInstallCode,INSTALL_CODE_LEN);

    Zstackapi_bdbGenerateInstallCodeCRCReq(uiAppEntity,&zstack_bdbGenerateInstallCodeCRCReq,
                                                 &zstack_bdbGenerateInstallCodeCRCRsp);

    uiInstallCode[INSTALL_CODE_LEN] = zstack_bdbGenerateInstallCodeCRCRsp.CRC  & 0xFF;
    uiInstallCode[INSTALL_CODE_LEN + 1] = zstack_bdbGenerateInstallCodeCRCRsp.CRC  >> 8;
  }

  if (keys & UI_KEY_SW_5_PRESSED)
  {
#if (ZG_BUILD_COORDINATOR_TYPE)
    if (uiCurrentState == UI_STATE_SET_INST_CODE_ADDR_1)
    {
      uiCurrentState = UI_STATE_SET_INST_CODE_ADDR_0;
    }
    else
#endif
    {
      uiCurrentState = UI_STATE_SET_INST_CODE_0;
    }

    uiCursorLine = 0xFF;
  }
}

/*********************************************************************
 * @fn          uiActionConfigureEnables
 *
 * @brief       State-machine action for toggling various configurations on and off
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionConfigureEnables(uint16_t keys)
{
#if defined ( BDB_TL_TARGET )
  static uint32_t TouchlinkTimeoutIncrements = TOUCHLINK_TIMEOUT_INCREMENTS_INITIAL;
#endif

  switch (uiCurrentState)
  {
#ifdef BDB_TL_TARGET
    case UI_STATE_TOUCHLINK_TARGET:
      if (keys & UI_KEY_SW_1_PRESSED)
      {
        if (uiTouchlinkTargetTimeout < TOUCHLINK_TARGET_PERPETUAL - TouchlinkTimeoutIncrements)
        {
          uiTouchlinkTargetTimeout += TouchlinkTimeoutIncrements;
        }
        else
        {
          uiTouchlinkTargetTimeout = TOUCHLINK_TARGET_PERPETUAL;
        }
      }
      if (keys & UI_KEY_SW_3_PRESSED)
      {
        if (uiTouchlinkTargetTimeout > 0 + TouchlinkTimeoutIncrements)
        {
          uiTouchlinkTargetTimeout -= TouchlinkTimeoutIncrements;
        }
        else
        {
          uiTouchlinkTargetTimeout = 0;
        }
      }
      break;
    case UI_STATE_TOUCHLINK_STEALING:
    {
      zstack_bdbTouchlinkGetAllowStealingRsp_t zstack_bdbTouchlinkGetAllowStealingRsp;
      zstack_bdbTouchlinkSetAllowStealingReq_t zstack_bdbTouchlinkSetAllowStealingReq;

      Zstackapi_bdbTouchlinkGetAllowStealingReq(uiAppEntity,&zstack_bdbTouchlinkGetAllowStealingRsp);
      zstack_bdbTouchlinkSetAllowStealingReq.allowStealing = !(zstack_bdbTouchlinkGetAllowStealingRsp.allowStealing);

      Zstackapi_bdbTouchlinkSetAllowStealingReq(uiAppEntity, &zstack_bdbTouchlinkSetAllowStealingReq );
    }
      break;
#endif
#ifdef BDB_TL_INITIATOR
    case UI_STATE_TOUCHLINK_INITIATOR:
      *(pUiSelectedBdbComissioningModes) ^= BDB_COMMISSIONING_MODE_INITIATOR_TL;
      break;
#endif
    case UI_STATE_NETWORK_FORMATION:
      *(pUiSelectedBdbComissioningModes) ^= BDB_COMMISSIONING_MODE_NWK_FORMATION;
      break;
    case UI_STATE_NETWORK_STEERING:
      *(pUiSelectedBdbComissioningModes) ^= BDB_COMMISSIONING_MODE_NWK_STEERING;
      break;
    case UI_STATE_FINDING_AND_BINDING:
      *(pUiSelectedBdbComissioningModes) ^= BDB_COMMISSIONING_MODE_FINDING_BINDING;
      break;
  }
}

/*********************************************************************
 * @fn          uiActionResetToFactoryNew
 *
 * @brief       State-machine action for resetting the device to factory new
 *
 * @param       keys - the keypress code that triggered the call to this function
 *
 * @return      none
 */
static void uiActionResetToFactoryNew(uint16_t keys)
{
  uiResetInitiated = TRUE;
  if(uiAppFNReset != NULL)
  {
      uiAppFNReset();
  }
  Zstackapi_bdbResetLocalActionReq(uiAppEntity);
}

/*********************************************************************
 * @fn          UI_MainStateMachine
 *
 * @brief       This is the main UI state machine engine.
 *              This function should be called from the application's key handler, and also from its event-loop
 *              for processing the key-repress event.
 *
 * @param       current_keys - keys received from UART interface
 *
 * @return      none
 */
void UI_MainStateMachine( uint8_t keys )
{
  uint8_t uiSavedCursorLine = uiCursorLine;
  bool uiRefreshIsRequired = FALSE;

  if (uiResetInitiated)
  {
    return;
  }

  uiCursorLine = 0xFF;

  if ((keys & UI_KEY_SW_2_PRESSED) && (uiStates[uiCurrentState].next_state != UI_STATE_UNCHANGED))
  {
    if (uiStates[uiCurrentState].next_state == UI_STATE_DEFAULT_MOVE)
    {
      uiCurrentState++;
    }
    else
    {
      uiCurrentState = uiStates[uiCurrentState].next_state;
    }

    uiRefreshIsRequired = TRUE;
  }
  else if ((keys & UI_KEY_SW_4_PRESSED) && (uiStates[uiCurrentState].prev_state != UI_STATE_UNCHANGED))
  {
    if (uiStates[uiCurrentState].prev_state == UI_STATE_DEFAULT_MOVE)
    {
      uiCurrentState--;
    }
    else
    {
      uiCurrentState = uiStates[uiCurrentState].prev_state;
    }

    uiRefreshIsRequired = TRUE;
  }
  else if ((keys & uiStates[uiCurrentState].keys_mask) && (uiStates[uiCurrentState].state_func != NULL))
  {
    uiStates[uiCurrentState].state_func(keys);

    uiRefreshIsRequired = TRUE;
  }

  if (uiRefreshIsRequired)
  {
    UI_Refresh();
  }
  else
  {
    uiCursorLine = uiSavedCursorLine;
  }
}

/*********************************************************************
 * @fn          UI_UpdateComissioningStatus
 *
 * @brief       Update the comissioning status to be displayed when the respective menu-screen is selected.
 *              This function should be called from the application, from within the function registered to
 *              be notified about these events.
 *
 * @param       bdbCommissioningModeMsg - comissioning mode and state information
 *
 * @return      none
 */
void UI_UpdateComissioningStatus(bdbCommissioningModeMsg_t *bdbCommissioningModeMsg)
{
  uiCommissioningIsInitializing = FALSE;

  switch(bdbCommissioningModeMsg->bdbCommissioningMode)
  {
    case BDB_COMMISSIONING_FORMATION:
      uiCommissioningStateStr = "NF";
      if ((bdbCommissioningModeMsg->bdbCommissioningStatus == BDB_COMMISSIONING_SUCCESS) && (uiCommissioningNetworkConnectionStr == NULL))
      {
        uiCommissioningNetworkConnectionStr = "FORM";
      }
      break;
    case BDB_COMMISSIONING_NWK_STEERING:
      uiCommissioningStateStr = "NS";
      if ((bdbCommissioningModeMsg->bdbCommissioningStatus == BDB_COMMISSIONING_SUCCESS) && (uiCommissioningNetworkConnectionStr == NULL))
      {
        uiCommissioningNetworkConnectionStr = "JOIN";
      }
      break;
    case BDB_COMMISSIONING_FINDING_BINDING:
      uiCommissioningStateStr = "FB";
      break;
    case BDB_COMMISSIONING_INITIALIZATION:
      uiCommissioningStateStr = "IN";
      break;
#if ZG_BUILD_ENDDEVICE_TYPE
    case BDB_COMMISSIONING_PARENT_LOST:
      if(bdbCommissioningModeMsg->bdbCommissioningStatus == BDB_COMMISSIONING_NETWORK_RESTORED)
      {
        uiCommissioningStateStr = "--";
      }
      else
      {
        uiCommissioningStateStr = "PL";
      }
      break;
#endif
#if ( BDB_TOUCHLINK_CAPABILITY_ENABLED == TRUE )
    case BDB_COMMISSIONING_TOUCHLINK:
      uiCommissioningStateStr = "TL";
      if ((bdbCommissioningModeMsg->bdbCommissioningStatus == BDB_COMMISSIONING_SUCCESS) && (uiCommissioningNetworkConnectionStr == NULL))
      {
        uiCommissioningNetworkConnectionStr = "TCHL";
      }
      break;
#endif
  }

  if ( ((bdbCommissioningModeMsg->bdbCommissioningStatus != BDB_COMMISSIONING_IN_PROGRESS) &&
        (bdbCommissioningModeMsg->bdbCommissioningStatus != BDB_COMMISSIONING_FB_TARGET_IN_PROGRESS) &&
        (bdbCommissioningModeMsg->bdbCommissioningStatus != BDB_COMMISSIONING_FB_INITITATOR_IN_PROGRESS))
        && (bdbCommissioningModeMsg->bdbRemainingCommissioningModes == 0) )
  {
    uiCommissioningStateStr = "--";
  }

  UI_Refresh();

}


void Hal_Ui_HW_WriteChar(uint8_t line, uint8_t col, char text);


/*********************************************************************
 * @fn          UI_Refresh
 *
 * @brief       Update the UI display.
 *              This function should be called by the application when handling the display-auto-refresh event, and
 *              also whenever an action that may change the information being displayed is executed.
 *
 * @param       none
 *
 * @return      none
 */
void UI_Refresh( void )
{
  uint8_t i;
  uint8_t *xad;
  bool uiAutoUpdateRequired = FALSE;
  uint8_t PermitJoinDuration;

  //Initialize the buffer
  memset(LineUARTBuf,' ',sizeof(LineUARTBuf));

  //Set the pointers to the strings
  line[0] = &LineUARTBuf[1];
  line[1] = &LineUARTBuf[UI_DISPLAY_LINE_LENGTH + 3];
  line[2] = &LineUARTBuf[(UI_DISPLAY_LINE_LENGTH * 2) + 5];


#ifdef HAL_BOARD_CC2538
  for (i = 0; i < UI_DISPLAY_CC2538_PREFIX_COUNT; i++)
  {
    LineBuf[0][i] = ' ';
    LineBuf[1][i] = ' ';
    LineBuf[2][i] = ' ';
  }
#endif

  Timer_setTimeout(uiAppClkHandle,  UI_CURSOR_REFRESH_INTERVAL);

  Timer_stop(&uiAppClkStruct);

  if (uiStates == uiAppStatesMain)
  {
    if (uiCurrentState == UI_STATE_BACK_FROM_APP_MENU)
    {
      uiConstStrOver(line[2], "<     BACK     >");
    }
    else
    {
      uiRefresh(uiCurrentState, line);

      //Copy the application specific messages
      OsalPort_memcpy(&LineUARTBuf[1],line[0],UI_DISPLAY_LINE_LENGTH);
      OsalPort_memcpy(&LineUARTBuf[UI_DISPLAY_LINE_LENGTH + 3],line[1],UI_DISPLAY_LINE_LENGTH);
      OsalPort_memcpy(&LineUARTBuf[(UI_DISPLAY_LINE_LENGTH * 2) + 5],line[2],UI_DISPLAY_LINE_LENGTH);
    }
  }
  else
  {
    switch (uiCurrentState)
    {
      case UI_STATE_FINDING_AND_BINDING:
          if(*(pUiSelectedBdbComissioningModes) & BDB_COMMISSIONING_MODE_FINDING_BINDING)
          {
              uiConstStrOver(line[1], "ENABLED");
          }
          else
          {
              uiConstStrOver(line[1], "DISABLED");
          }
        uiConstStrOver(line[2], "<FINDNG+BINDNG >");
        break;
      case UI_STATE_BACK_FROM_CONFIGURE:
        uiConstStrOver(line[2], "<     BACK     >");
        break;
      case UI_STATE_BACK_FROM_INSTALL_CODE:
        uiConstStrOver(line[2], "<     BACK     >");
        break;
      case UI_STATE_NETWORK_FORMATION:
          if(*(pUiSelectedBdbComissioningModes) & BDB_COMMISSIONING_MODE_NWK_FORMATION)
          {
              uiConstStrOver(line[1], "ENABLED");
          }
          else
          {
              uiConstStrOver(line[1], "DISABLED");
          }
        uiConstStrOver(line[2], "<NWK FORMATION >");
        break;
      case UI_STATE_NETWORK_STEERING:
          if(*(pUiSelectedBdbComissioningModes) & BDB_COMMISSIONING_MODE_NWK_STEERING)
          {
              uiConstStrOver(line[1], "ENABLED");
          }
          else
          {
              uiConstStrOver(line[1], "DISABLED");
          }
        uiConstStrOver(line[2], "< NWK STEERING >");
        break;
#ifdef BDB_TL_TARGET
      case UI_STATE_TOGGLE_TL_TARGET:
        {
          uint32_t temp_u32;
          zstack_bdbTouchLinkTargetGetTimerRsp_t zstack_bdbTouchLinkTargetGetTimerRsp;

          Zstackapi_bdbTouchLinkTargetGetTimerReq(uiAppEntity,&zstack_bdbTouchLinkTargetGetTimerRsp);

          temp_u32 = (zstack_bdbTouchLinkTargetGetTimerRsp.Time);
          if (temp_u32 == 0)
          {
            uiConstStrOver(line[1], "DISABLED        ");
          }
          else if (temp_u32 == TOUCHLINK_TARGET_PERPETUAL)
          {
            uiConstStrOver(line[1], "ENABLED FOREVER ");
          }
          else
          {
            uiConstStrOver(line[1], "ENABLED (     s)");
            uiUintToString( temp_u32 / 1000 + ((temp_u32 % 1000) > 0 ? 1 : 0), line[1] + 9, 10, 5, FALSE, FALSE); //note: timeouts longer than 65535 seconds will not display correctly

            Timer_setTimeout(uiAppClkHandle, uiAutoRefreshInterval);
            Timer_start(&uiAppClkStruct);
          }
        }
        uiConstStrOver(line[2], "< T.L. TARGET  >");
        break;
      case UI_STATE_TOUCHLINK_TARGET:
        if (uiTouchlinkTargetTimeout == 0)
        {
          uiConstStrOver(line[1], "DISABLED        ");
        }
        else if (uiTouchlinkTargetTimeout == TOUCHLINK_TARGET_PERPETUAL)
        {
          uiConstStrOver(line[1], "Enable (forever)");
        }
        else
        {
          uiConstStrOver(line[1], "Enable For     s");
          uiUintToString( uiTouchlinkTargetTimeout / 1000 + ((uiTouchlinkTargetTimeout % 1000) > 0 ? 1 : 0), line[1] + 10, 10, 5, FALSE, FALSE); //note: timeouts longer than 65535 seconds will not display correctly
          line[1][16] = 0;
        }
        uiConstStrOver(line[2], "<T.L. TRGT TIME>");
        break;
      case UI_STATE_TOUCHLINK_STEALING:
        {
            zstack_bdbTouchlinkGetAllowStealingRsp_t AllowStealingRsp;
            Zstackapi_bdbTouchlinkGetAllowStealingReq(uiAppEntity,&AllowStealingRsp);
            if(AllowStealingRsp.allowStealing)
            {
                uiConstStrOver(line[1], "ENABLED");
            }
            else
            {
                uiConstStrOver(line[1], "DISABLED");
            }
            uiConstStrOver(line[2], "<T.L. STEALING >");
        }
        break;
#endif
#ifdef BDB_TL_INITIATOR
      case UI_STATE_TOUCHLINK_INITIATOR:
          if(*(pUiSelectedBdbComissioningModes) & BDB_COMMISSIONING_MODE_INITIATOR_TL)
          {
              uiConstStrOver(line[1], "ENABLED");
          }
          else
          {
              uiConstStrOver(line[1], "DISABLED");
          }
        uiConstStrOver(line[2], "<T.L. INITIATOR>");
        break;
#endif
      case UI_STATE_CONFIGURE:
        uiConstStrOver(line[2], "<  CONFIGURE   >");
        break;
      case UI_STATE_RESET_TO_FACTORY_NEW:
        if (uiResetInitiated)
        {
          uiConstStrOver(line[0], "Resetting,");
          uiConstStrOver(line[1], "Please wait...");
        }
        else
        {
          uiConstStrOver(line[2], "< RESET TO FN  >");
        }
        break;
      case UI_STATE_DEFAULT:
        OsalPort_memcpy(line[0], uiAppTitleStr,UI_DISPLAY_LINE_LENGTH);
        uiConstStrOver(line[1], "press E for help");
        uiConstStrOver(line[2], "<     HELP     >");
        break;
      case UI_STATE_STATE_HELP:
#ifdef HAL_BOARD_CC2538
        line[0] = "< > to move     ";
        line[1] = "^ v change value";
#else
        uiConstStrOver(line[0], "A D to move     ");
        uiConstStrOver(line[1], "W S change value");
#endif
        uiConstStrOver(line[2], "E to execute    ");
        break;
      case UI_STATE_COMMISSION:
        if (uiCommissioningIsInitializing)
        {
          uiConstStrOver(line[0], "Please wait...  ");
        }
        else
        {
          uiConstStrOver(line[0], "xx              "); // TL/NF/NS/FB/-- TCHL/FORM/JOIN count/CLOSE

          OsalPort_memcpy(line[0] + 0, uiCommissioningStateStr, 2);

          if ((uiNwkStateShadow != zstack_DevState_DEV_END_DEVICE) && (uiNwkStateShadow != zstack_DevState_DEV_ZB_COORD) && (uiNwkStateShadow != zstack_DevState_DEV_ROUTER))
          {
            uiConstStrOver(line[0] + 3, "NotOnNwk");
          }
          else
          {
            if (uiCommissioningNetworkConnectionStr != NULL)
            {
              OsalPort_memcpy(line[0] + 3, uiCommissioningNetworkConnectionStr, 4);
              line[0][7] = ' ';
            }

            if ((uiNwkStateShadow == zstack_DevState_DEV_ZB_COORD) || (uiNwkStateShadow == zstack_DevState_DEV_ROUTER))
            {
              PermitJoinDuration = NLME_GetRemainingPermitJoiningDuration();

              if (PermitJoinDuration > 0)
              {
                uiConstStrOver(line[0] + 8, "Open");
                uiUintToString(PermitJoinDuration, line[0] + 12, 10, 3, TRUE, FALSE);
                uiAutoUpdateRequired = TRUE;
              }
              else
              {
                uiConstStrOver(line[0] + 8, "CLOSED");
              }
            }
           }
        }

        uiConstStrOver(line[1], "Id000 Srch000/00"); // IDENTIFYING, SEARCHING   - IDFYcnt SRCHcnt

        uiUintToString(*pUiIdentifyTimeAttribute, line[1] + 2, 10, 3, TRUE, FALSE);

        zstack_bdbGetFBInitiatorStatusRsp_t zstack_bdbGetFBInitiatorStatusRsp;

        Zstackapi_bdbGetFBInitiatorStatusReq(uiAppEntity, &zstack_bdbGetFBInitiatorStatusRsp);

        uiUintToString(zstack_bdbGetFBInitiatorStatusRsp.RemainingTime, line[1] + 10, 10, 3, TRUE, FALSE);
        uiUintToString(FBMatchesFound, line[1] + 14, 10, 2, TRUE, FALSE);

        if ((*pUiIdentifyTimeAttribute > 0) || (zstack_bdbGetFBInitiatorStatusRsp.RemainingTime > 0) || (uiCommissioningStateStr != "--"))
        {
          uiAutoUpdateRequired = TRUE;
        }

        uiConstStrOver(line[2], "<  COMMISSION  >");

        if (uiAutoUpdateRequired)
        {
          Timer_setTimeout(uiAppClkHandle,  UI_AUTO_REFRESH_INTERVAL_ON_COMISSIONING_SCREEN );

          Timer_start(&uiAppClkStruct);
        }
        break;
      case UI_STATE_SET_INST_CODE:
        uiConstStrOver(line[2], "<ADD INSTL CODE>");
        break;
      case UI_STATE_SET_INST_CODE_0:
        uiArrayToString(uiInstallCode, line[0], 16, TRUE);
        uiArrayToString(uiInstallCode + 8, line[1], 16, TRUE);
        line[0][16] = 0;
        line[1][16] = 0;
        uiConstStrOver(line[2], "<SET INSTL CODE>");
        break;
#if (ZG_BUILD_COORDINATOR_TYPE)
      case UI_STATE_SET_INST_CODE_ADDR_0:
        uiArrayToString(uiInstallCodeAddr, line[0],16, FALSE);
        line[0][16] = 0;
        uiConstStrOver(line[2], "<SET I.C. ADDR >");
        break;
#endif
      case UI_STATE_SET_INST_CODE_DONE:
        uiConstStrOver(line[0], "Last status:");

        switch (uiAddInstallCodeLastStatus)
        {
          case STATUS_UNKNOWN:
            uiConstStrOver(line[1], "---");
            break;
          case ZSuccess:
            uiConstStrOver(line[1], "SUCCESS");
            break;
          default:
            uiConstStrOver(line[1], "ERROR (0x  )    ");
            uiUintToString( uiAddInstallCodeLastStatus, line[1] + 9, 16, 2, TRUE, FALSE);
            break;
        }
        uiConstStrOver(line[2], "<APLY INST CODE>");
        break;
#if (ZG_BUILD_COORDINATOR_TYPE)
      case UI_STATE_SET_INST_CODE_ADDR_1:
        uiArrayToString(uiInstallCodeAddr, line[0],16, FALSE);
        line[0][16] = 0;
        uiConstStrOver(line[2], "       I.C. ADDR");
        break;
#endif
      case UI_STATE_SET_INST_CODE_1:
        uiArrayToString(uiInstallCode, line[0], 16, TRUE);
        uiArrayToString(uiInstallCode + 8, line[1], 16, TRUE);
        line[0][16] = 0;
        line[1][16] = 0;
        uiConstStrOver(line[2], "       INST CODE");
        uiArrayToString( uiInstallCode + 16, line[2], 4, TRUE);
        break;
      case UI_STATE_APP_SPECIFIC_MENU:
        uiConstStrOver(line[2], "<   APP MENU   >");
        break;
      case UI_STATE_SET_PRI_CHANEL_MASK_0:
        uiConstStrOver(line[0], "1234567890123456");
        uiCreateChannelMaskString(bdbAttributes.bdbPrimaryChannelSet, line[1]);
        uiConstStrOver(line[2], "<PRI CHANL MASK>");
        break;
      case UI_STATE_SET_SEC_CHANEL_MASK_0:
        uiConstStrOver(line[0], "1234567890123456");
        uiCreateChannelMaskString(bdbAttributes.bdbSecondaryChannelSet, line[1]);
        uiConstStrOver(line[2], "<SEC CHANL MASK>");
        break;
      case UI_STATE_SET_PRI_CHANEL_MASK_1:
        uiConstStrOver(line[0], "1234567890123456");
        uiCreateChannelMaskString(bdbAttributes.bdbPrimaryChannelSet, line[1]);
        uiConstStrOver(line[2], " PRI CHANL MASK ");
        break;
      case UI_STATE_SET_SEC_CHANEL_MASK_1:
        uiConstStrOver(line[0], "1234567890123456");
        uiCreateChannelMaskString(bdbAttributes.bdbSecondaryChannelSet, line[1]);
        uiConstStrOver(line[2], " SEC CHANL MASK ");
        break;
      case UI_STATE_SET_PAN_ID_0:
        if (zgConfigPANID == 0xFFFF)
        {
          uiConstStrOver(line[0], "    0xFFFF (any)");
        }
        else
        {
          uiConstStrOver(line[0], "    0x          ");
          uiUintToString( zgConfigPANID, line[0] + 6, 16, 4, TRUE, FALSE);
        }
        uiConstStrOver(line[2], "<    PAN ID    >");
        break;
      case UI_STATE_SET_PAN_ID_1:
        if (zgConfigPANID == 0xFFFF)
        {
          uiConstStrOver(line[0], "    0xFFFF (any)");
        }
        else
        {
          uiConstStrOver(line[0], "    0x          ");
          uiUintToString( zgConfigPANID, line[0] + 6, 16, 4, TRUE, FALSE);
        }
        uiConstStrOver(line[2], "     PAN ID     ");
        break;
#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
      case UI_STATE_GPP_COMMISSION:
        uiConstStrOver(line[0], "    GP PROXY    ");
        uiConstStrOver(line[1], " COMMISSIONING  ");

        if(gp_GetProxyCommissioningMode() == TRUE)
        {
          if(gppCommissioningTimeout > 0)
          {
            uiConstStrOver(line[2], "TIME ");
            //there is a timeout, so lets print it
            uiUintToString(gppCommissioningTimeout, line[2] + 5, 10, 3, TRUE, FALSE);
          }
          else
          {
            //No time, then it is just enabled
            uiConstStrOver(line[2], "    ENABLED     ");
          }
        }
        else
        {
          gppCommissioningTimeout = 0;
          uiConstStrOver(line[2], "    DISABLED    ");
        }

      break;
#endif
      case UI_STATE_INFO:
        // Display the extended address.
        xad = aExtendedAddress + Z_EXTADDR_LEN - 1;

        for (i = 0; i < Z_EXTADDR_LEN * 2; xad--)
        {
          uint8_t ch;
          ch = (*xad >> 4) & 0x0F;
          line[0][i++] = ch + (( ch < 10 ) ? '0' : '7');
          ch = *xad & 0x0F;
          line[0][i++] = ch + (( ch < 10 ) ? '0' : '7');
        }

        line[0][Z_EXTADDR_LEN*2] = '\0';

        uiConstStrOver( line[1], "nxxxx cxx axxxx ");

        uiUintToString( _NIB.nwkPanId, line[1] + 1 , 16, 4, TRUE, FALSE);
        line[1][5] = ' ';

        uiUintToString( _NIB.nwkLogicalChannel, (void *)(line[1] + 7), 10, 2, TRUE, FALSE );
        line[1][9] = ' ';

        uiUintToString( _NIB.nwkDevAddress, line[1] + 11, 16, 4, TRUE, FALSE );

        uiConstStrOver(line[2], "          <INFO>");
        switch (uiNwkStateShadow)
        {
          case zstack_DevState_DEV_ZB_COORD:
            uiConstStrOver(line[2], "ZC");
            break;
          case zstack_DevState_DEV_ROUTER:
            if (APSME_IsDistributedSecurity())
            {
              uiConstStrOver(line[2], "ZR DIST");
            }
            else
            {
              uiConstStrOver(line[2], "ZR CENT");
            }
            break;
          case zstack_DevState_DEV_END_DEVICE:
            uiConstStrOver(line[2], "ZED p");
            uiUintToString( _NIB.nwkCoordAddress, line[2] + 5, 16, 4, TRUE, FALSE );
            break;
          default:
#if ZG_BUILD_COORDINATOR_TYPE
            uiConstStrOver(line[2], "ZC (   )");
            uiUintToString( uiNwkStateShadow, line[2] + 4, 10, 3, TRUE, FALSE );
#elif ZG_BUILD_RTRONLY_TYPE
            uiConstStrOver(line[2], "ZR (   )");
            uiUintToString( uiNwkStateShadow, line[2] + 4, 10, 3, TRUE, FALSE );
#elif ZG_BUILD_ENDDEVICE_TYPE
            uiConstStrOver(line[2], "ZED (   )");
            uiUintToString( uiNwkStateShadow, line[2] + 5, 10, 3, TRUE, FALSE );
#else
            uiConstStrOver(line[2], "(   )");
            uiUintToString( uiNwkStateShadow, line[2] + 1, 10, 3, TRUE, FALSE );
#endif
            break;
        }

        break;
      default:
        break;
    }
  }

  if (uiCursorLine != 0xFF)
  {
    if (uiCursorState == 0)
    {
      line[uiCursorLine][uiCursorCol] = UI_CURSOR_CHAR; //note: the cursor must point to a writeable memory location, i.e. not const
    }
    uiCursorState ^= 1;

    Timer_setTimeout(uiAppClkHandle, UI_CURSOR_REFRESH_INTERVAL);
    Timer_start(&uiAppClkStruct);
  }

#ifdef HAL_BOARD_CC2538
  for (i = 0; i < 3; i++)
  {
    if (line[i] != LineBuf[i] + UI_DISPLAY_CC2538_PREFIX_COUNT)
    {
      OsalPort_memcpy(LineBuf[i] + UI_DISPLAY_CC2538_PREFIX_COUNT, line[i], UI_DISPLAY_LINE_LENGTH + 1);
    }

    line[i] = LineBuf[i];
  }
#endif


#ifdef BOARD_DISPLAY_USE_UART

  //Clear screen
  LineUARTBuf[0] = 0x0C;
  //Set the newline and carriage return
  LineUARTBuf[UI_DISPLAY_LINE_LENGTH + 1] =    '\n';
  LineUARTBuf[UI_DISPLAY_LINE_LENGTH + 2] =    '\r';

  LineUARTBuf[UI_DISPLAY_LINE_LENGTH*2 + 3] =  '\n';
  LineUARTBuf[UI_DISPLAY_LINE_LENGTH*2 + 4 ] = '\r';

  display_writeString(LineUARTBuf, sizeof(LineUARTBuf), UI_DISPLAY_LINE_1);

#endif


#ifdef DEBUG_UI_REFRESH
  {
    static bool toggle=TRUE;
    Hal_Ui_HW_WriteChar(3,15,(toggle = !toggle) ? '>' : ' ');
  }
#endif


}

/*********************************************************************
 * @fn          uiProcessPermitJoin
 *
 * @brief       Update the Network-Joining status LED
 *
 * @param       Req - Pointer to The time left for joining, or 0 if the device is closed for joining
 *
 * @return      none
 */
void uiProcessPermitJoin( zstack_devPermitJoinInd_t *Req )
{
  UI_Refresh();
}





#ifdef BOARD_DISPLAY_USE_UART

static void UartWriteCallback(UART_Handle handle, void *buf, size_t count)
{

}



static void UartReadCallback(UART_Handle handle, void *buf, size_t size)
{
  // Make sure we received all expected bytes
  if (size == totalRxBytes)
  {
    //If cleared, then read it
    if(uartTxBuffer[0] == 0)
    {
        // Copy bytes from RX buffer to TX buffer
        for(size_t i = 0; i < size; i++)
        {
          uartTxBuffer[i] = ((uint8_t*)buf)[i];
        }
    }

    //Set the event
    *uiAppEvents |= uiUartProcessEvent;

    Semaphore_post(uiAppSem);

  }
  else
  {
    // Handle error or call to UART_readCancel()
  }
}


void zclSampleAppsUI_ProcessUART(void)
{
    switch(uartTxBuffer[0])
    {
        case 'w':
        case 'W':
            UI_MainStateMachine(UI_KEY_SW_1_PRESSED);
        break;
        case 'd':
        case 'D':
            UI_MainStateMachine(UI_KEY_SW_2_PRESSED);
        break;
        case 's':
        case 'S':
            UI_MainStateMachine(UI_KEY_SW_3_PRESSED);
        break;
        case 'a':
        case 'A':
            UI_MainStateMachine(UI_KEY_SW_4_PRESSED);
        break;
        case 'e':
        case 'E':
            UI_MainStateMachine(UI_KEY_SW_5_PRESSED);
        break;
    }

    //Clear the buffer
    uartTxBuffer[0] = 0;

    UART_read(Uhandle, uartRxBuffer, sizeof(uartRxBuffer));
}


#endif




/*******************************************************************************
 * @fn      zclSampleAppsUI_ProcessRefreshTimeoutCallback
 *
 * @brief   Timeout handler function
 *
 * @param   a0 - ignored
 *
 * @return  none
 */
static void zclSampleAppsUI_ProcessRefreshTimeoutCallback(UArg a0)
{
    (void)a0; // Parameter is not used

    //Assign the UI event
    *uiAppEvents |= uiAutoRefreshEvent;

    // Wake up the application thread when it waits for clock event
    Semaphore_post(uiAppSem);
}

#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
/*******************************************************************************
 * @fn      zclSampleAppsUI_ProcessGPPUpdateTimeoutCallback
 *
 * @brief   Timeout handler function
 *
 * @param   a0 - ignored
 *
 * @return  none
 */
static void zclSampleAppsUI_ProcessGPPUpdateTimeoutCallback(UArg a0)
{
  (void)a0; // Parameter is not used

  //Assign the gpp event
  *uiAppEvents |= uigppProcessEvent;

  // Wake up the application thread when it waits for clock event
  Semaphore_post(uiAppSem);
}
#endif

/*******************************************************************************
 * @fn      zclSampleApps_initializeClocks
 *
 * @brief   Initialize Clocks
 *
 * @param   none
 *
 * @return  none
 */
static void zclSampleApps_initializeClocks(void)
{
    // Construct the timer used to switch back to the main screen
    uiAppClkHandle = Timer_construct(
    &uiAppClkStruct,
    zclSampleAppsUI_ProcessRefreshTimeoutCallback,
    UI_CURSOR_REFRESH_INTERVAL,
    0,
    false,
    0);
#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
    // Construct the timer used to update gpp timeout screen
    uiGppClkHandle = Timer_construct(
    &uiGppClkStruct,
    zclSampleAppsUI_ProcessGPPUpdateTimeoutCallback,
    GPP_TIMEOUT_UPDATE_INTERVAL,
    0,
    false,
    0);
#endif
}



/*********************************************************************
 * @fn          UI_Init
 *
 * @brief       Initialize the user-interface module.
 *              This function must be called by the application during its initializatin.
 *
 * @param       zclSampleAppEvents - the events process by the sample application
 *              zclSampleAppSem - Semaphore to post the events in the application thread
 *              ui_auto_refresh_event_value - the event mask (single bit) for using as the ui-auto-update event.
 *              uart_process_event_value - the event mask (single bit) for processing UART characters (only used if BOARD_DISPLAY_USE_UART is enabled)
 *              ui_IdentifyTimeAttribute_value - a pointer to the identify attribute's value.
 *              app_title_value - a pointer to a string, holding the title of the sample app.
 *              _UiRefresh - a pointer to a fuction to refresh the UI when displaying the application-
 *                specific screens.
 *              _uiAppStatesMain - a pointer to the application-specific state-table.
 *
 * @return      none
 */
void UI_Init(uint8_t  zclSampleApp_Entity, uint32_t *zclSampleAppEvents, Semaphore_Handle zclSampleAppSem,
             uint16_t ui_auto_refresh_event_value, uint16_t uart_process_event_value, uint16_t * ui_IdentifyTimeAttribute_value,
             char * app_title_value, uiRefresh_t _uiRefresh, const uiState_t _uiAppStatesMain[],uiAppFNReset_t _uiAppFNReset,
             uint8_t * ui_bdb_commissioning_modes)
{
  zstack_bdbStartCommissioningReq_t zstack_bdbStartCommissioningReq;

  uiAppEntity = zclSampleApp_Entity;
  uiAppSem = zclSampleAppSem;
  uiAppEvents = zclSampleAppEvents;
  uiAutoRefreshEvent = ui_auto_refresh_event_value;
  uiUartProcessEvent = uart_process_event_value;
  pUiIdentifyTimeAttribute = ui_IdentifyTimeAttribute_value;
  uiAppTitleStr = app_title_value;
  uiRefresh = _uiRefresh;
  uiAppStatesMain = _uiAppStatesMain;
  uiAppFNReset = _uiAppFNReset;
  pUiSelectedBdbComissioningModes = ui_bdb_commissioning_modes;

#if defined(BOARD_DISPLAY_USE_UART)
    /* Enable System_printf(..) UART output */
    UART_init();
    UART_Params_init(&uartParams);
    uartParams.baudRate = 115200;
    uartParams.writeMode      = UART_MODE_CALLBACK;
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.writeCallback = UartWriteCallback;
    uartParams.readMode      = UART_MODE_CALLBACK;
    uartParams.readDataMode  = UART_DATA_BINARY;
    uartParams.readCallback  = UartReadCallback;
    Uhandle = UART_open(Board_UART0, &uartParams);
    UART_read(Uhandle, uartRxBuffer, sizeof(uartRxBuffer));
    //UartPrintf_init();
#endif /* BOARD_DISPLAY_USE_UART */

  zclSampleApps_initializeClocks();

  zstack_bdbStartCommissioningReq.commissioning_mode = 0;

  Zstackapi_bdbStartCommissioningReq(uiAppEntity,&zstack_bdbStartCommissioningReq);

}

/*********************************************************************
 * @fn      uiProcessTouchlinkTargetEnable
 *
 * @brief   Update the UI when the touchlink target functionality is enabled or disabled
 *
 * @param   enable - the current state of the touchlink target (0 - disabled; 1 - enabled)
 *
 * @return  none
 */
#ifdef BDB_TL_TARGET
void uiProcessTouchlinkTargetEnable( uint8_t enable )
{
  UI_Refresh();
}
#endif

/*********************************************************************
 * @fn      uiProcessIdentifyTimeChange
 *
 * @brief   Update the UI when the identify time attribute's value has changed
 *
 * @param   endpoint - the endpoints which had its identify time attribute changed
 *
 * @return  none
 */
void uiProcessIdentifyTimeChange( uint8_t *endpoint )
{
  if ( *pUiIdentifyTimeAttribute > 0 )
  {
    Board_Led_control(UI_LED_DEVICE_STATUS, board_led_state_BLINKING);

  }
  else
  {
      switch(uiNwkStateShadow)
      {
        case zstack_DevState_DEV_END_DEVICE:
        case zstack_DevState_DEV_ROUTER:
        case zstack_DevState_DEV_ZB_COORD:
          Board_Led_control(UI_LED_DEVICE_STATUS, board_led_state_ON);
        break;
        default:
          Board_Led_control(UI_LED_DEVICE_STATUS, board_led_state_OFF);
        break;
      }
  }
}

/*********************************************************************
 * @fn      uiProcessBindNotification
 *
 * @brief   Uopdate the UI when a bind is added
 *
 * @param   data - information about the new binding entry
 *
 * @return  none
 */
void uiProcessBindNotification( bdbBindNotificationData_t *data )
{
  //note: 'data' argument contains information about the new binding, which can be useful for the user's application

  if (FBMatchesFound < 255)
  {
    FBMatchesFound++;
  }
}

/*********************************************************************
 * @fn      UI_DeviceStateUpdated
 *
 * @brief   Update the UI when the device state has changed
 *
 * @param   NwkState - the new network-state of the device
 *
 * @return  none
 */
void UI_DeviceStateUpdated(zstack_devStateChangeInd_t *Req)
{
  uiNwkStateShadow = Req->state;

  if ( *pUiIdentifyTimeAttribute == 0 )
  {
  switch (Req->state)
  {
    case zstack_DevState_DEV_ZB_COORD:
    case zstack_DevState_DEV_ROUTER:
    case zstack_DevState_DEV_END_DEVICE:
        Board_Led_control(UI_LED_DEVICE_STATUS, board_led_state_ON);
      break;
    default:
        Board_Led_control(UI_LED_DEVICE_STATUS, board_led_state_OFF);
      break;
      }
  }

  UI_Refresh();
}


/*******************************************************************************
 * @fn      display_writeString
 *
 * @brief   Stub function for user interface
 *
 * @param   keys  - keys that were pressed
 *
 * @return  void
 */
#if defined (BOARD_DISPLAY_USE_UART)
static void display_writeString(void * buffer, size_t size, uint8_t line)
{
  //Error if no buffer
  if((Uhandle != NULL) && (buffer != NULL) )
  {
    UART_write(Uhandle, (const void *)buffer, size);
  }
}
#endif

#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
/*******************************************************************************
 * @fn      UI_SetGPPCommissioningMode
 *
 * @brief   Interface to set the GP Proxy commissioning mode
 *
 * @param   isEntering  - TRUE if commissioning mode is being enabled, False otherwise
 * @param   hasTime     - TRUE if is entering the commissioning with a timeout, false otherwise
 * @param   time        - Time in seconds for which the device is in commissioning mode
 *
 * @return  void
 */
void UI_SetGPPCommissioningMode( zstack_gpCommissioningMode_t *Req )
{
  gp_SetProxyCommissioningMode(Req->isEntering);

  if(Req->hasTime)
  {
    if(Req->time > 0)
    {
      gppCommissioningTimeout = Req->time;
      Timer_stop(&uiGppClkStruct);
      Timer_setTimeout(uiGppClkHandle,1000);
      Timer_start(&uiGppClkStruct);
    }
    else
    {
      gp_SetProxyCommissioningMode(FALSE);
    }
  }
  else
  {
    gppCommissioningTimeout = 0;
    Timer_stop(&uiGppClkStruct);
    Timer_setTimeout(uiGppClkHandle, 0);
  }

  UI_Refresh();
}

/*******************************************************************************
 * @fn      UI_ProcessGPPCommissioningTimeout
 *
 * @brief   Process the timer to update the UI timeout
 *
 * @param   none
 *
 * @return  void
 */
void UI_ProcessGPPCommissioningTimeout(void)
{
    zstack_gpCommissioningMode_t commissioningMode;
    if(gppCommissioningTimeout > 0)
    {
        gppCommissioningTimeout--;
        commissioningMode.isEntering = TRUE;
        commissioningMode.hasTime = TRUE;
    }
    else
    {
      commissioningMode.isEntering = FALSE;
      commissioningMode.hasTime = FALSE;
    }
    commissioningMode.time = gppCommissioningTimeout;
    UI_SetGPPCommissioningMode(&commissioningMode);
}

#endif

#else


/*
 * User-Interface intialization
 */
void UI_Init(uint8_t  zclSampleApp_Entity, uint32_t *zclSampleAppEvents, Semaphore_Handle zclSampleAppSem,
             uint16_t ui_auto_refresh_event_value, uint16_t uart_process_event_value, uint16_t * ui_IdentifyTimeAttribute_value,
             char * app_title_value, uiRefresh_t _uiRefresh, const uiState_t _uiAppStatesMain[],uiAppFNReset_t _uiAppFNReset,
             uint8_t * ui_bdb_commissioning_modes)
{
}

/*
 * Process commissioning status change
 */
void UI_UpdateComissioningStatus(bdbCommissioningModeMsg_t *bdbCommissioningModeMsg)
{
}

/*
 * Update the UI display
 */
void UI_Refresh( void )
{
}

/*
 * Main UI state-machine
 */
void UI_MainStateMachine( uint8_t keys )
{
}

/*
 * Menu-item action for returning from the application-specific menu to the common menu
 */
void UI_ActionBackFromAppMenu(uint16_t keys)
{
}

/*
 * Process a change in the device's network-state
 */
void UI_DeviceStateUpdated(zstack_devStateChangeInd_t *Req)
{
}

#ifdef BDB_TL_TARGET
/*
 * @brief   Uopdate the UI when the touchlink target functionality is enabled or disabled
 */
void uiProcessTouchlinkTargetEnable( uint8_t enable )
{
}
#endif


void uiProcessIdentifyTimeChange( uint8_t *endpoint )
{
}

void uiProcessPermitJoin( zstack_devPermitJoinInd_t *Req )
{
}

void zclSampleAppsUI_ProcessUART(void)
{
}

void uiProcessBindNotification( bdbBindNotificationData_t *data )
{
}

#endif


