 /**************************************************************************************************
  Filename:       zcl_sampleapps_ui.h
  Revised:        $Date: 2016-08-01 08:38:22 -0700 (Thu, 19 Jun 2014) $
  Revision:       $Revision: 39101 $

  Description:    This file contains the generic Sample Application User Interface.


  Copyright 2006-2014 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

#ifndef ZCL_SAMPLEAPPS_UI_H
#define ZCL_SAMPLEAPPS_UI_H

#include "zstackmsg.h"
#include "zstackapi.h"
#include <string.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * MACROS
 */
#define UI_STATE_BACK_FROM_APP_MENU 0

#define UI_STATE_DEFAULT_MOVE 254
#define UI_STATE_UNCHANGED 255

#define UI_KEY_SW_1_PRESSED 0x01
#define UI_KEY_SW_2_PRESSED 0x02
#define UI_KEY_SW_3_PRESSED 0x04
#define UI_KEY_SW_4_PRESSED 0x08
#define UI_KEY_SW_5_PRESSED 0x10

#define UI_LED_APP HAL_LED_1

/*********************************************************************
 * TYPEDEFS
 */
typedef struct
{
  uint8_t next_state; // state to move to when [RIGHT] is pressed
  uint8_t prev_state; // state to move to when [LEFT] is changed
  uint16_t keys_mask; // pressing the keys enabled in this bitmask will execute the state function (state_func)
  void (*state_func)(uint16_t); // the function to execute when any of the keys in keys_mask is pressed/released (according to the mask)
} uiState_t;

typedef void (* uiRefresh_t)(uint8_t uiCurrentState, char * line[3]);
typedef void (* uiAppFNReset_t)(void);
/*********************************************************************
 * FUNCTIONAL MACROS
 */

//uiConstStrOver - copy src string (must be a constant quoted string) onto dst,
//without copying the null character that terminates src
#define uiConstStrOver(dst,src) do {OsalPort_memcpy(dst, src, sizeof(src) - 1);} while (0)

//uiConstStrCpy - copy src string (must be a constant quoted string) onto dst,
//including copying the null character that terminates src
#define uiConstStrCpy(dst,src) do {OsalPort_memcpy(dst, src, sizeof(src));} while (0)

/*********************************************************************
* FUNCTIONS
*/

/*
 * User-Interface intialization
 */
void UI_Init(uint8_t  zclSampleApp_Entity, uint32_t *zclSampleAppEvents, Semaphore_Handle zclSampleAppSem,
             uint16_t ui_auto_refresh_event_value, uint16_t uart_process_event_value, uint16_t * ui_IdentifyTimeAttribute_value,
             char * app_title_value, uiRefresh_t _uiRefresh, const uiState_t _uiAppStatesMain[],uiAppFNReset_t _uiAppFNReset,
             uint8_t * ui_bdb_commissioning_modes);

/*
 * Process commissioning status change
 */
void UI_UpdateComissioningStatus(bdbCommissioningModeMsg_t *bdbCommissioningModeMsg);

/*
 * Update the UI display
 */
void UI_Refresh( void );

/*
 * Main UI state-machine
 */
void UI_MainStateMachine( uint8_t keys );

/*
 * Menu-item action for returning from the application-specific menu to the common menu
 */
void UI_ActionBackFromAppMenu(uint16_t keys);

/*
 * State-machine action for executing the application-specific sub-menus *
 */
void UI_ActionAppSpecificMenu(uint16_t keys);

/*
 * State-machine action for starting comissioning
 */
void UI_ActionStartComissioning(uint16_t keys);

/*
 * External action to configuring the PAN ID
 */
void UI_ActionConfigurePanId(uint16_t panID);

/*
 * External action to configuring the Pri Chan Mask
 */
void UI_actionConfigureChannels(uint32_t channelMask);

/*
 * Process a change in the device's network-state
 */
void UI_DeviceStateUpdated(zstack_devStateChangeInd_t *Req);

#ifdef BDB_TL_TARGET
/*
 * @brief   Uopdate the UI when the touchlink target functionality is enabled or disabled
 */
void uiProcessTouchlinkTargetEnable( uint8_t enable );
#endif


void uiProcessIdentifyTimeChange( uint8_t *endpoint );

void uiProcessPermitJoin( zstack_devPermitJoinInd_t *Req );

void zclSampleAppsUI_ProcessUART(void);

void uiProcessBindNotification( bdbBindNotificationData_t *data );

#if !defined (DISABLE_GREENPOWER_BASIC_PROXY) && (ZG_BUILD_RTR_TYPE)
/*
 * @brief   Interface to set the GP Proxy commissioning mode
 */
void UI_SetGPPCommissioningMode( zstack_gpCommissioningMode_t *Req );
/*
 * @brief   Process the timer to update the UI timeout
 */
void UI_ProcessGPPCommissioningTimeout(void);
#endif

#ifdef __cplusplus
}
#endif

#endif
